@@ -4,7 +4,7 @@ import os
 config_text = 'site_name: My Docs\n'
 index_text = """# Welcome to Docums
 
-For full documentation visit [Docums](https://khanhduy1407.github.io/docums).
+For full documentation visit [Docums](https://documsapp.github.io/docums).
 
 ## Commands
 
