@@ -1,7 +1,3 @@
-import json
-
-import markupsafe
-
 try:
     from jinja2 import pass_context as contextfilter
 except ImportError:
@@ -10,10 +6,6 @@ except ImportError:
 from docums.utils import normalize_url
 
 
-def tojson(obj, **kwargs):
-    return markupsafe.Markup(json.dumps(obj, **kwargs))
-
-
 @contextfilter
 def url_filter(context, value):
     """ A Template filter to normalize URLs. """
